@@ -103,7 +103,7 @@ Rboolean progress_along_Inspect(SEXP x,
 void* progress_along_Dataptr(SEXP x, Rboolean writeable) {
   SEXP data1 = R_altrep_data1(x);
   if (writeable) {
-    return DATAPTR(data1);
+    return (void*) INTEGER(data1);
   } else {
     return (void*) DATAPTR_RO(data1);
   }
@@ -123,7 +123,7 @@ int progress_along_Elt(SEXP x, R_xlen_t i) {
     Rf_defineVar(PROTECT(Rf_install("current")), PROTECT(ScalarReal((int) i)), bar);
     cli__current_progress_bar = bar;
     SEXP show_after = clic__find_var(bar, Rf_install("show_after"));
-    if (now > REAL(show_after)[0]) DATAPTR(cli__disable_gc);
+    if (now > REAL(show_after)[0]) INTEGER(cli__disable_gc);
     UNPROTECT(2);
   }
   return (int) (i + 1);
