@@ -233,7 +233,7 @@ test_that_cli(configs = "ansi", "styling pieces of a dl", {
   expect_snapshot(local({
     cli_div(
       theme = list(
-        .dt = list(after = " -> "),
+        .dt = list(postfix = " -> "),
         .dd = list(color = "blue")
       )
     )
@@ -243,7 +243,8 @@ test_that_cli(configs = "ansi", "styling pieces of a dl", {
 
 test_that("cli_dl edge cases", {
   # invalid input
-  expect_error(
+  expect_snapshot(
+    error = TRUE,
     cli_dl("foo", "must be a named character vector")
   )
 
@@ -252,3 +253,12 @@ test_that("cli_dl edge cases", {
     cli_dl(c(abc = "foo", empty = "", def = "bar"))
   )
 })
+
+test_that("cli_dl label style", {
+  expect_snapshot(local({
+    cli_dl(c(
+      "{.code code}" = "{.code this is code too}",
+      "{.str strin}" = "{.url https://x.com}"
+    ))
+  }))
+})
