@@ -1,13 +1,13 @@
 
-start_app()
-on.exit(stop_app(), add = TRUE)
+start_app(.auto_close = TRUE)
 
 test_that_cli("add/remove/list themes", {
-  local_rng_version("3.3.0")
+  withr::local_rng_version("3.3.0")
   set.seed(24)
 
+  start_app(.auto_close = TRUE)
+
   id <- default_app()$add_theme(list(".green" = list(color = "green")))
-  on.exit(default_app()$remove_theme(id), add = TRUE)
   expect_true(id %in% names(default_app()$list_themes()))
 
   expect_snapshot({
@@ -115,7 +115,7 @@ start_app()
 on.exit(stop_app(), add = TRUE)
 
 test_that_cli(configs = "ansi", "NULL will undo background color", {
-  if (packageVersion("testthat") <= "3.1.1") skip("needs newer testthat")
+  skip_if_not_installed("testthat", "3.1.2")
   expect_snapshot(local({
     cli_alert("{.emph {.code this has bg color}}")
     cli_div(theme = list(span = list("background-color" = NULL)))
